@@ -9,7 +9,7 @@
 ### This code was originally written to package up BitPim but has been
 ### refactored to build almost any Python program.
 ###
-### $Id: makedist.py 4413 2007-09-28 18:22:42Z djpham $
+### $Id: makedist.py 4653 2008-07-31 21:47:19Z djpham $
 
 
 # Runs on Linux, Windows and Mac
@@ -671,6 +671,10 @@ if __name__=='__main__':
 
     if _thisversion:
         pkg.setversion(_thisversion)
+    # rebuild depedence files
+    if hasattr(pkg, 'build_dependences'):
+        sys.path.append(os.path.join(os.getcwd(), 'packaging'))
+        pkg.build_dependences()
     # do a sanity check
     sanitycheck()  # ours
     if hasattr(pkg, 'sanitycheck'):
