@@ -5,8 +5,9 @@
 ### This program is free software; you can redistribute it and/or modify
 ### it under the terms of the BitPim license as detailed in the LICENSE file.
 ###
-### $Id: calendarentryeditor.py 3895 2007-01-12 01:34:45Z djpham $
+### $Id: calendarentryeditor.py 4397 2007-09-13 01:26:23Z djpham $
 
+from __future__ import with_statement
 import calendar
 import copy
 import datetime
@@ -40,6 +41,8 @@ class RepeatEditor(pb_editor.DirtyUIBase):
     _daily_option_index=0
     _weekly_option_index=1
     _monthly_option_index=2
+    _weekly_wkst_str=(None, 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun')
+    _weekly_wkst_idx=(7, 1, 2, 3, 4, 5, 6)
     
     def __init__(self, parent, _):
         global widgets_list
@@ -93,6 +96,13 @@ class RepeatEditor(pb_editor.DirtyUIBase):
             wx.EVT_CHECKBOX(self, self._wl_dow[i].GetId(), self.OnDirtyUI)
             hbs.Add(self._wl_dow[i], 0, wx.LEFT|wx.TOP, 5)
         vbs.Add(hbs, 0, wx.LEFT, 5)
+        hbs=wx.BoxSizer(wx.HORIZONTAL)
+        hbs.Add(wx.StaticText(self, -1, 'Week starts on:'), 0, wx.LEFT, 10)
+        self._wl_wkst=wx.ComboBox(self, -1, value=self._dow[0],
+                                  choices=self._dow, style=wx.CB_READONLY)
+        wx.EVT_COMBOBOX(self, self._wl_wkst.GetId(), self.OnDirtyUI)
+        hbs.Add(self._wl_wkst, 0, wx.LEFT, 5)
+        vbs.Add(hbs, 0, wx.TOP, 10)
         self._option_bs.Add(vbs, 0, wx.LEFT, 5)
         self._weekly_option_bs=vbs
         # monthly option widgets
@@ -169,6 +179,7 @@ class RepeatEditor(pb_editor.DirtyUIBase):
             for i in range(len(self._wl_dow)):
                 b=((1<<i)&dow_mask)!=0
                 self._wl_dow[i].SetValue(b)
+            self._wl_wkst.SetValue(self._weekly_wkst_str[data.weekstart])
         elif rt==data.monthly:
             self._repeat_option_rb.SetSelection(3)
             self._ml_interval.SetValue(`data.interval2`)
@@ -221,6 +232,7 @@ class RepeatEditor(pb_editor.DirtyUIBase):
             if self._wl_dow[i].GetValue():
                 m=m|(1<<i)
         r.dow=m
+        r.weekstart=self._weekly_wkst_idx[self._wl_wkst.GetSelection()]
     def _get_monthly_options(self, r):
         r.repeat_type=r.monthly
         try:
@@ -359,10 +371,8 @@ class GeneralEditor(pb_editor.DirtyUIBase):
         end=datetime.datetime(*self._w['end'].GetValue())
         if start>end:
             # scold the user
-            dlg=wx.MessageDialog(self, "End date and time is before start!", "Time Travel Attempt Detected",
-                                wx.OK|wx.ICON_EXCLAMATION)
-            dlg.ShowModal()
-            dlg.Destroy()
+            guihelper.MessageDialog(self, "End date and time is before start!", "Time Travel Attempt Detected",
+                                    wx.OK|wx.ICON_EXCLAMATION)
             # move focus
             self._w['end'].SetFocus()
             return False
@@ -462,8 +472,9 @@ class CategoryEditor(pb_editor.DirtyUIBase):
         hs.Fit(self)
 
     def OnManageCategories(self, _):
-        dlg=pb_editor.CategoryManager(self)
-        dlg.ShowModal()
+        with guihelper.WXDialogWrapper(pb_editor.CategoryManager(self),
+                                       True):
+            pass
 
     def OnUpdateCategories(self, msg):
         cats=msg.data[:]
@@ -966,16 +977,15 @@ class Editor(wx.Dialog):
         return self._AskAboutRecurringEvent("Change recurring event?", "Do you want to change all the recurring events, or just this one?", "Change")
 
     def _AskAboutRecurringEvent(self, caption, text, prefix):
-        dlg=RecurringDialog(self, caption, text, prefix)
-        res=dlg.ShowModal()
-        dlg.Destroy()
-        if res==dlg.ID_THIS:
-            return self.ANSWER_THIS
-        if res==dlg.ID_ALL:
-            return self.ANSWER_ORIGINAL
-        if res==dlg.ID_CANCEL:
-            return self.ANSWER_CANCEL
-        assert False
+        with guihelper.WXDialogWrapper(RecurringDialog(self, caption, text, prefix),
+                                       True) as (dlg, res):
+            if res==dlg.ID_THIS:
+                return self.ANSWER_THIS
+            if res==dlg.ID_ALL:
+                return self.ANSWER_ORIGINAL
+            if res==dlg.ID_CANCEL:
+                return self.ANSWER_CANCEL
+            assert False
 #------------------------------------------------------------------------------    
 # We derive from wxPanel not the control directly.  If we derive from
 # wx.MaskedTextCtrl then all hell breaks loose as our {Get|Set}Value
