@@ -5,7 +5,7 @@
 ### This program is free software; you can redistribute it and/or modify
 ### it under the terms of the BitPim license as detailed in the LICENSE file.
 ###
-### $Id: common_calendar.py 4376 2007-08-26 16:55:46Z djpham $
+### $Id: common_calendar.py 4675 2008-08-12 21:52:12Z djpham $
 
 "Common stuff for the Calendar Import functions"
 
@@ -312,14 +312,18 @@ class FilterDialogBase(wx.Dialog):
         wx.EVT_CHECKBOX(self, self._end_date_chkbox.GetId(), self.OnCheckBox)
         wx.EVT_CHECKBOX(self, self.__cat_chkbox.GetId(), self.OnCheckBox)
         wx.EVT_RADIOBOX(self, self.__alarm_setting.GetId(), self.OnAlarmSetting)
+        # Listen to changes in ringtones list
+        pubsub.subscribe(self.OnRingtoneUpdates, pubsub.ALL_RINGTONES)
         # all done
         self.SetSizer(bs)
         self.SetAutoLayout(True)
         bs.Fit(self)
 
+    def __del__(self):
+        pubsub.unsubscribe(self.OnRingtoneUpdates)
+
     def ShowModal(self):
         # request ringtones from 
-        pubsub.subscribe(self.OnRingtoneUpdates, pubsub.ALL_RINGTONES)
         wx.CallAfter(pubsub.publish, pubsub.REQUEST_RINGTONES) # make the call once we are onscreen
         return wx.Dialog.ShowModal(self)
 
