@@ -7,13 +7,13 @@
 ### This program is free software; you can redistribute it and/or modify
 ### it under the terms of the BitPim license as detailed in the LICENSE file.
 ###
-### $Id: fileview.py 4226 2007-05-09 21:21:04Z djpham $
+### $Id: fileview.py 4412 2007-09-28 01:28:56Z djpham $
 
 
 ###
 ### File viewer
 ###
-
+from __future__ import with_statement
 import os
 import copy
 import cStringIO
@@ -144,8 +144,10 @@ class MediaEntry(object):
     def _get_timestamp(self):
         return self._data.get('timestamp', None)
     def _set_timestamp(self, v):
-        if v is not None and not isinstance(v, int):
-            raise TypeError('duration property is an int arg')
+        if v is not None:
+            if not isinstance(v, (int, float)):
+                raise TypeError('timestamp property is an int arg')
+            v=int(v)
         self._set_or_del('timestamp', v)
     timestamp=property(fget=_get_timestamp, fset=_set_timestamp)
 
@@ -356,6 +358,8 @@ class FileView(wx.Panel, widgets.BitPimWidget):
         wx.EVT_KEY_DOWN(self.aggdisp, self.OnKeyDown)
         wx.EVT_KEY_UP(self.aggdisp, self.OnKeyUp)
         self.tb.Realize()
+        pubsub.subscribe(self.OnMediaInfo, pubsub.REQUEST_MEDIA_INFO)
+        pubsub.subscribe(self.OnMediaOpen, pubsub.REQUEST_MEDIA_OPEN)
 
     def OnIdle(self, _):
         "Save out changed data"
@@ -500,9 +504,8 @@ class FileView(wx.Panel, widgets.BitPimWidget):
                 self.OnRefresh()
                 wx.EndBusyCursor()
 
-    def OnLaunch(self, _):
-        wx.BeginBusyCursor()
-        item=self.GetSelectedItems()[0]
+    def _launch(self, item):
+        # Open/Launch the specified item
         me=self._data[self.database_key][item.key]
         fname=self._gettempfile(me)
         if guihelper.IsMac():
@@ -514,7 +517,10 @@ class FileView(wx.Panel, widgets.BitPimWidget):
             wx.Bell()
         else:
             wx.Execute(cmd, wx.EXEC_ASYNC)
-        wx.EndBusyCursor()
+
+    @guihelper.BusyWrapper
+    def OnLaunch(self, _):
+        self._launch(self.GetSelectedItems()[0])
 
     if True: # guihelper.IsMSWindows() or guihelper.IsGtk():
         # drag-and-drop files should work on all platforms
@@ -559,8 +565,9 @@ class FileView(wx.Panel, widgets.BitPimWidget):
         self.thetimer.Start(1750, wx.TIMER_ONE_SHOT)
 
     def OnTooltipTimer(self, _):
-        if self._in_context_menu or not self.show_thumbnail:
-            # we're putting up a context menu, forget this
+        if self._in_context_menu or not self.show_thumbnail or \
+           wx.GetApp().critical.isSet():
+            # we're putting up a context menu or main app is busy, forget this
             return
         # see if we have moved
         if self.abs_mouse_pos!=wx.GetMousePosition():
@@ -628,24 +635,24 @@ class FileView(wx.Panel, widgets.BitPimWidget):
             ext=getext(items[0].name)
             if ext=="": ext="*"
             else: ext="*."+ext
-            dlg=wx.FileDialog(self, "Save item", wildcard=ext, defaultFile=items[0].name, style=wx.SAVE|wx.OVERWRITE_PROMPT|wx.CHANGE_DIR)
-            if dlg.ShowModal()==wx.ID_OK:
-                file(dlg.GetPath(), "wb").write(self._data[items[0].datakey][items[0].key].mediadata)
-                if self._data[items[0].datakey][items[0].key].timestamp!=None:
-                    os.utime(dlg.GetPath(), (self._data[items[0].datakey][items[0].key].timestamp, 
-                                             self._data[items[0].datakey][items[0].key].timestamp))
-            dlg.Destroy()
+            with guihelper.WXDialogWrapper(wx.FileDialog(self, "Save item", wildcard=ext, defaultFile=items[0].name, style=wx.SAVE|wx.OVERWRITE_PROMPT|wx.CHANGE_DIR),
+                                           True) as (dlg, retcode):
+                if retcode==wx.ID_OK:
+                    file(dlg.GetPath(), "wb").write(self._data[items[0].datakey][items[0].key].mediadata)
+                    if self._data[items[0].datakey][items[0].key].timestamp!=None:
+                        os.utime(dlg.GetPath(), (self._data[items[0].datakey][items[0].key].timestamp, 
+                                                 self._data[items[0].datakey][items[0].key].timestamp))
         else:
-            dlg=wx.DirDialog(self, "Save items to", style=wx.DD_DEFAULT_STYLE|wx.DD_NEW_DIR_BUTTON)
-            if dlg.ShowModal()==wx.ID_OK:
-                for item in items:
-                    fname=item.name.encode(media_codec)
-                    fname=os.path.join(dlg.GetPath(), basename(fname))
-                    file(fname, 'wb').write(self._data[item.datakey][item.key].mediadata)
-                    if self._data[item.datakey][item.key].timestamp!=None:
-                        os.utime(fname, (self._data[item.datakey][item.key].timestamp, 
-                                         self._data[item.datakey][item.key].timestamp))
-            dlg.Destroy()
+            with guihelper.WXDialogWrapper(wx.DirDialog(self, "Save items to", style=wx.DD_DEFAULT_STYLE|wx.DD_NEW_DIR_BUTTON),
+                                           True) as (dlg, retcode):
+                if retcode==wx.ID_OK:
+                    for item in items:
+                        fname=item.name.encode(media_codec)
+                        fname=os.path.join(dlg.GetPath(), basename(fname))
+                        file(fname, 'wb').write(self._data[item.datakey][item.key].mediadata)
+                        if self._data[item.datakey][item.key].timestamp!=None:
+                            os.utime(fname, (self._data[item.datakey][item.key].timestamp, 
+                                             self._data[item.datakey][item.key].timestamp))
 
     if guihelper.IsMSWindows():
         def OnCopy(self, _):
@@ -985,10 +992,10 @@ class FileView(wx.Panel, widgets.BitPimWidget):
         return True
 
     def OnAdd(self, _=None):
-        dlg=wx.FileDialog(self, "Choose files", style=wx.OPEN|wx.MULTIPLE, wildcard=self.wildcard)
-        if dlg.ShowModal()==wx.ID_OK:
-            self.OnAddFiles(dlg.GetPaths())
-        dlg.Destroy()
+        with guihelper.WXDialogWrapper(wx.FileDialog(self, "Choose files", style=wx.OPEN|wx.MULTIPLE, wildcard=self.wildcard),
+                                       True) as (dlg, retcode):
+            if retcode==wx.ID_OK:
+                self.OnAddFiles(dlg.GetPaths())
 
     def CanRename(self):
         return len(self.GetSelectedItems())==1
@@ -1000,33 +1007,33 @@ class FileView(wx.Panel, widgets.BitPimWidget):
                # either none or more than 1 items selected
                return
         old_name=items[0].name
-        dlg=wx.TextEntryDialog(self, "Enter a new name:", "Item Rename",
-                               old_name)
-        if dlg.ShowModal()==wx.ID_OK:
-            new_name=dlg.GetValue()
-            if len(new_name) and new_name!=old_name:
-                items[0].name=new_name
-                items[0].RenameInIndex(new_name)
-                pubsub.publish(pubsub.MEDIA_NAME_CHANGED,
-                               data={ pubsub.media_change_type: self.media_notification_type,
-                                      pubsub.media_old_name: old_name,
-                                      pubsub.media_new_name: new_name })
-        dlg.Destroy()
+        with guihelper.WXDialogWrapper(wx.TextEntryDialog(self, "Enter a new name:", "Item Rename",
+                                                          old_name),
+                                       True) as (dlg, retcode):
+            if retcode==wx.ID_OK:
+                new_name=dlg.GetValue()
+                if len(new_name) and new_name!=old_name:
+                    items[0].name=new_name
+                    items[0].RenameInIndex(new_name)
+                    pubsub.publish(pubsub.MEDIA_NAME_CHANGED,
+                                   data={ pubsub.media_change_type: self.media_notification_type,
+                                          pubsub.media_old_name: old_name,
+                                          pubsub.media_new_name: new_name })
           
     def OnAddFiles(self,_):
-        raise Exception("not implemented")
+        raise NotImplementedError
 
     def OnReplace(self, _=None):
         items=self.GetSelectedItems()
         if len(items)!=1:
                # either none or more than 1 items selected
                return
-        dlg=wx.FileDialog(self, "Choose file",
-                          style=wx.OPEN, wildcard=self.wildcard)
-        if dlg.ShowModal()==wx.ID_OK:
-            self.ReplaceContents(items[0].name, items[0].origin, dlg.GetPath())
-            items[0].Refresh()
-        dlg.Destroy()
+        with guihelper.WXDialogWrapper(wx.FileDialog(self, "Choose file",
+                                                     style=wx.OPEN, wildcard=self.wildcard),
+                                       True) as (dlg, retcode):
+            if retcode==wx.ID_OK:
+                self.ReplaceContents(items[0].name, items[0].origin, dlg.GetPath())
+                items[0].Refresh()
 
     def get_media_name_from_filename(self, filename, newext=''):
         path,filename=os.path.split(filename)
@@ -1106,6 +1113,24 @@ class FileView(wx.Panel, widgets.BitPimWidget):
     def GetHelpID(self):
         return self.helpid
 
+    def OnMediaInfo(self, msg):
+        # return the list of strings (lines) describing this item
+        client, name, origin=msg.data
+        for _item in self.GetAllItems():
+            if (origin is None or _item.origin==origin) and \
+               _item.name==name:
+                    pubsub.publish(pubsub.RESPONSE_MEDIA_INFO,
+                                   { 'client': client,
+                                     'canopen': bool(guihelper.GetOpenCommand(_item.mimetypes, _item.name)),
+                                     'desc': _item.lines })
+    def OnMediaOpen(self, msg):
+        # Launch the specified item name
+        name, origin=msg.data
+        for _item in self.GetAllItems():
+            if (origin is None or _item.origin==origin) and \
+               _item.name==name:
+                return self._launch(_item)
+
 class FileViewDisplayItem(object):
 
     datakey="Someone forgot to set me"
