@@ -7,7 +7,7 @@
 ### This program is free software; you can redistribute it and/or modify
 ### it under the terms of the BitPim license as detailed in the LICENSE file.
 ###
-### $Id: package.py 4574 2008-01-15 02:12:19Z djpham $
+### $Id: package.py 4654 2008-07-31 21:47:50Z djpham $
 
 # This file provides the packaging definitions used by the buildrelease system
 
@@ -24,7 +24,7 @@ def sanitycheck():
     print "=== Sanity check ==="
 
     print "svn location",
-    if not "$HeadURL: https://bitpim.svn.sourceforge.net/svnroot/bitpim/releases/1.0.5/src/package.py $".split(":",1)[1].strip().startswith("https://bitpim.svn.sourceforge.net"):
+    if not "$HeadURL: https://bitpim.svn.sourceforge.net/svnroot/bitpim/releases/1.0.6/src/package.py $".split(":",1)[1].strip().startswith("https://bitpim.svn.sourceforge.net"):
         raise Exception("Needs to be checked out from https://bitpim.svn.sourceforge.net")
     print "  OK"
 
@@ -297,6 +297,14 @@ def resourcefilter(srcfilename, destfilename):
         return srcfilename, destfilename
     return None
 
+def build_dependences():
+    # build modules
+    import buildmodules
+    # rebuild all the prototocol (*.p) files
+    import protogen
+    for f in glob.glob("src/phones/*.p"):
+        protogen.processfile(f, f+"y")
+
 def finalize(destdir):
     if sys.platform=='win32':
         for f in ("w9xpopen.exe",):
