@@ -6,7 +6,7 @@
 ### This program is free software; you can redistribute it and/or modify
 ### it under the terms of the BitPim license as detailed in the LICENSE file.
 ###
-### $Id: __init__.py 4572 2008-01-15 01:58:54Z djpham $
+### $Id: __init__.py 4678 2008-08-13 23:46:56Z djpham $
 
 import helpids
 
@@ -49,6 +49,11 @@ _phonedata= { 'LG-AX8600': { 'module': 'com_lgax8600',
                             'brand': b_lg,
                             'helpid': helpids.ID_PHONE_LGC2000,
                             },
+              'LG-LX570 (Musiq)': { 'module': 'com_lglx570',
+                                    'carrier': [c_sprint],
+                                    'brand': b_lg,
+                                    'helpid': helpids.ID_PHONE_LGLX570,
+                            },
               'LG-UX5000': { 'module': 'com_lgux5000',
                              'brand': b_lg,
                              'carrier': [c_vzw],
@@ -149,51 +154,71 @@ _phonedata= { 'LG-AX8600': { 'module': 'com_lgax8600',
                              'brand': b_lg,
                              'helpid': None,
                              },
-              'LG-VX8500': { 'module': 'com_lgvx8500',
-                             'carrier': [c_vzw],
-                             'brand': b_lg,
-                             'helpid': helpids.ID_PHONE_LGVX8500,
-                             },
-              'LG-VX8550': { 'module': 'com_lgvx8550',
-                             'carrier': [c_vzw],
-                             'brand': b_lg,
-                             'helpid': None,
-                             },
+              'LG-VX8500 (Chocolate)': { 'module': 'com_lgvx8500',
+                                         'carrier': [c_vzw],
+                                         'brand': b_lg,
+                                         'helpid': helpids.ID_PHONE_LGVX8500,
+                                         },
+              'LG-VX8550 (Chocolate 2)': { 'module': 'com_lgvx8550',
+                                           'carrier': [c_vzw],
+                                           'brand': b_lg,
+                                           'helpid': None,
+                                           },
+              'LG-VX8560 (Chocolate 3)': { 'module': 'com_lgvx8560',
+                                           'carrier': [c_vzw],
+                                           'brand': b_lg,
+                                           'helpid': helpids.ID_PHONE_LGVX8560,
+                                           },
               'LG-VX8600': { 'module': 'com_lgvx8600',
                              'carrier': [c_vzw],
                              'brand': b_lg,
                              'helpid': helpids.ID_PHONE_LGVX8600,
                              },
+              'LG-VX8610 (Decoy)': { 'module': 'com_lgvx8610',
+                                     'carrier': [c_vzw],
+                                     'brand': b_lg,
+                                     'helpid': helpids.ID_PHONE_LGVX8610,
+                             },
               'LG-VX8700': { 'module': 'com_lgvx8700',
                              'carrier': [c_vzw],
                              'brand': b_lg,
                              'helpid': helpids.ID_PHONE_LGVX8700,
                              },
-              'LG-VX8800': { 'module': 'com_lgvx8800',
-                             'carrier': [c_vzw],
-                             'brand': b_lg,
-                             'helpid': helpids.ID_PHONE_LGVX8800,
-                             },
+              'LG-VX8800 (Venus)': { 'module': 'com_lgvx8800',
+                                     'carrier': [c_vzw],
+                                     'brand': b_lg,
+                                     'helpid': helpids.ID_PHONE_LGVX8800,
+                                     },
               'LG-VX9400': { 'module': 'com_lgvx9400',
                              'carrier': [c_vzw],
                              'brand': b_lg,
                              'helpid': None,
                              },
+              'LG-VX9700 (Dare)': { 'module': 'com_lgvx9700',
+                                    'carrier': [c_vzw],
+                                    'brand': b_lg,
+                                    'helpid': helpids.ID_PHONE_LGVX9700,
+                                    },
               'LG-VX9800': { 'module': 'com_lgvx9800',
                              'carrier': [c_vzw],
                              'brand': b_lg,
                              'helpid': helpids.ID_PHONE_LGVX9800,
                              },
-              'LG-VX9900': { 'module': 'com_lgvx9900',
-                             'carrier': [c_vzw],
-                             'brand': b_lg,
-                             'helpid': helpids.ID_PHONE_LGVX9900,
-                             },
-              'LG-VX10000': { 'module': 'com_lgvx10000',
-                             'carrier': [c_vzw],
-                             'brand': b_lg,
-                             'helpid': helpids.ID_PHONE_LGVX10000,
-                             },
+              'LG-VX9900 (enV)': { 'module': 'com_lgvx9900',
+                                   'carrier': [c_vzw],
+                                   'brand': b_lg,
+                                   'helpid': helpids.ID_PHONE_LGVX9900,
+                                   },
+              'LG-VX9100 (enV 2)': { 'module': 'com_lgvx9100',
+                                     'carrier': [c_vzw],
+                                     'brand': b_lg,
+                                     'helpid': helpids.ID_PHONE_LGVX9100,
+                                     },
+              'LG-VX10000 (Voyager)': { 'module': 'com_lgvx10000',
+                                        'carrier': [c_vzw],
+                                        'brand': b_lg,
+                                        'helpid': helpids.ID_PHONE_LGVX10000,
+                                        },
               'LG-VI125': { 'module': 'com_lgvi125',
                             'carrier': [c_sprint],
                             'brand': b_lg,
@@ -247,6 +272,11 @@ _phonedata= { 'LG-AX8600': { 'module': 'com_lgax8600',
                            'brand': b_sanyo,
                            'helpid': helpids.ID_PHONE_SANYOOTHERS,
                            },
+              'SCP-8400': { 'module': 'com_sanyo8400',
+                            'carrier': [c_sprint],
+                            'brand': b_sanyo,
+                            'helpid': helpids.ID_PHONE_SANYOOTHERS,
+                            },
               'PM-8200': { 'module': 'com_sanyo8200',
                            'carrier': [c_sprint],
                            'brand': b_sanyo,
@@ -523,11 +553,16 @@ if __debug__:
                                       'carrier': [c_tmobileusa],
                                       'helpid': None,
                                       },
-                        'SCP-8400': { 'module': 'com_sanyo8400',
+                        'SPH-A640': { 'module': 'com_samsungspha640',
                                       'carrier': [c_sprint],
-                                      'brand': b_sanyo,
-                                      'helpid': helpids.ID_PHONE_SANYOOTHERS,
+                                      'brand': b_samsung,
+                                      'helpid': helpids.ID_PHONE_SAMSUNGOTHERS,
                                       },
+                        'LG-LX260 (Rumor)': {  'module': 'com_lglx260',
+                                       'carrier': [c_sprint],
+                                       'brand': b_lg,
+                                       'helpid': None,
+                                       }, 
                         })
 
 # update the module path
