@@ -5,7 +5,7 @@
 ### This program is free software; you can redistribute it and/or modify
 ### it under the terms of the BitPim license as detailed in the LICENSE file.
 ###
-### $Id: com_brew.py 4563 2008-01-11 21:49:45Z djpham $
+### $Id: com_brew.py 4664 2008-08-08 02:42:25Z djpham $
 
 """Implements the "Brew" filesystem protocol"""
 
@@ -368,9 +368,15 @@ class RealBrewProtocol:
 
     def mkdir(self, name):
         self.log("Making directory '"+name+"'")
+        if self.isdir(name):
+            raise BrewDirectoryExistsException
         req=p_brew.mkdirrequest()
         req.dirname=name
-        self.sendbrewcommand(req, p_brew.mkdirresponse)
+        try:
+            self.sendbrewcommand(req, p_brew.mkdirresponse)
+        except BrewDirectoryExistsException:
+            # sometime the phone returns this, which is OK
+            pass
 
     def mkdirs(self, directory):
         if len(directory)<1:
@@ -413,16 +419,20 @@ class RealBrewProtocol:
         self.rmdir(path)
 
     def exists(self, path):
-        # Return True if path refers to an existing path
-        # just try a list file command and see if it bombs out
-        req=p_brew.listfilerequest()
-        req.dirname=path
-        req.entrynumber=0
+        # Return True if this path (dir/file) exists
+        return bool(self.statfile(path))
+
+    def isdir(self, path):
+        # Return True if path refers to an existing dir
+        if not self.statfile(path):
+            # if it doesn't exist, bail
+            return False
+        # just try a list dirs command and see if it bombs out
+        req=p_brew.listdirectoriesrequest(dirname=path)
         try:
-            res=self.sendbrewcommand(req,p_brew.listfileresponse)
-        except BrewNoMoreEntriesException:
-            pass
-        except (BrewBadPathnameException, BrewNoSuchDirectoryException,
+            self.sendbrewcommand(req, p_brew.listdirectoriesresponse)
+        except (BrewCommandException, BrewBadPathnameException,
+                BrewNoSuchDirectoryException,
                 BrewAccessDeniedException):
             return False
         except:
@@ -431,6 +441,13 @@ class RealBrewProtocol:
             return False
         return True
 
+    def isfile(self, filename):
+        # return True if filename is a file
+        if not self.statfile(filename):
+            return False
+        # if it exists and not a dir, then it must be a file!
+        return not self.isdir(filename)
+
     def basename(self, path):
         # return the basename of the path, does not check on whether the path
         # exists.
@@ -450,11 +467,12 @@ class RealBrewProtocol:
 
     def join(self, *args):
         # join the dir/file components and return the full path name
-        return '/'.join([x.strip('/') for x in args])
+        return '/'.join([x.strip('/') for x in args if x])
 
     def listsubdirs(self, dir='', recurse=0):
         results={}
         self.log("Listing subdirs in dir: '"+dir+"'")
+        self.log("X recurse="+`recurse`)
 
         req=p_brew.listdirectoryrequest()
         req.dirname=dir
@@ -470,6 +488,7 @@ class RealBrewProtocol:
                     subdir=res.subdir
                 if len(dir):
                     subdir=dir+"/"+subdir
+                self.log("subdir="+subdir)
                 results[subdir]={ 'name': subdir, 'type': 'directory' }
             except BrewNoMoreEntriesException:
                 break
@@ -565,7 +584,7 @@ class RealBrewProtocol:
                     # invalid date - see SF bug #833517
                     results['date']=(0, '')
             return results
-        except BrewNoSuchFileException:
+        except (BrewCommandException, BrewNoSuchFileException):
             # File does not exist, bail
             return None
         except:
@@ -1002,6 +1021,41 @@ class RealBrewProtocol2(RealBrewProtocol):
             file_cache.add(file, node.get('date', [0])[0], data)
         return data
 
+    def getfilecontents2(self, filename, start, size):
+        # read and return data a block of data from the specified file
+        try:
+            block_size = self.protocolclass.BREW_READ_SIZE
+        except AttributeError:
+            block_size = p_brew.BREW_READ_SIZE
+        self.log("Getting file contents2 '"+filename+"'")
+        desc="Reading "+filename
+        data=cStringIO.StringIO()
+        handle=self.openfile(filename, p_brew.new_fileopen_mode_read)
+        _readsize=start+size
+        try:
+            read=start
+            counter=0
+            while True:
+                counter+=1
+                if counter%5==0:
+                    self.progress(read, _readsize, desc)
+                req=p_brew.new_readfilerequest()
+                req.handle=handle
+                req.bytes=block_size
+                req.position=read
+                res=self.sendbrewcommand(req, p_brew.new_readfileresponse)
+                if res.bytes:
+                    data.write(res.data)
+                    read+=res.bytes
+                else:
+                    break
+                if read>=_readsize:
+                    break
+        finally: # MUST close handle to file
+            self.closefile(handle)
+        self.progress(1,1,desc)
+        return data.getvalue()[:size]
+
     def listfiles(self, dir=''):
         self.log("Listing files in dir: '"+dir+"'")
         return self.getfilesystem(dir, recurse=0, directories=0)
@@ -1033,9 +1087,11 @@ class RealBrewProtocol2(RealBrewProtocol):
                     direntry=dir+"/"+res.entryname
                 else:
                     direntry=res.entryname
-                if files and res.type==0: # file or special file
-                    results[direntry]={ 'name': direntry, 'type': 'file',
-                                    'size': res.size }
+                if files and (res.type==0 or res.type == 0x0f): # file or special file
+                    if res.type == 0:
+                        results[direntry]={ 'name': direntry, 'type': 'file', 'size': res.size, 'special': False }
+                    elif res.type == 0x0f:
+                        results[direntry]={ 'name': direntry, 'type': 'file', 'size': res.size, 'special': True }
                     try:
                         if res.date<=0:
                             results[direntry]['date']=(0, "")
@@ -1043,7 +1099,7 @@ class RealBrewProtocol2(RealBrewProtocol):
                             results[direntry]['date']=(res.date, time.strftime("%x %X", time.localtime(res.date)))
                     except:
                         results[direntry]['date']=(0, "")
-                elif directories and res.type: # directory
+                elif directories and (res.type and res.type != 0x0f): # directory
                     results[direntry]={ 'name': direntry, 'type': 'directory' }
                     if recurse>0:
                         dirs[count]=direntry
@@ -1094,72 +1150,6 @@ else:
     BrewProtocol=RealBrewProtocol
 del phone_path
 
-##class BrewProtocol(RealBrewProtocol):
-##    """This is just a wrapper class that allows the manipulation between
-##    RealBrewProtocol and DebugBrewProtocol classes.
-##    """
-##    def __init__(self):
-##        RealBrewProtocol.__init__(self)
-##        # if the env var PHONE_FS is set, we're debugging!
-##        phone_path=os.environ.get('PHONE_FS', None)
-##        if __debug__ and phone_path:
-##            print 'Debug Phone File System:',phone_path
-##            # we probably need to do this only once for the whole class,
-##            # but what the heck!
-##            DebugBrewProtocol._fs_path=os.path.normpath(phone_path)
-##            for _attr in DebugBrewProtocol.exportedfuncs:
-##                setattr(self, _attr, getattr(DebugBrewProtocol, _attr))
-##            self._update_base_class(self.__class__)
-##            for _attr in DebugBrewProtocol.exportedfuncs:
-##                setattr(BrewProtocol, _attr, getattr(DebugBrewProtocol, _attr))
-##            # define BREW_FILE_SYSTEM=2 in protocol class to enable 
-##            # new brew filesystem support. Debug mode only
-####        elif __debug__ and getattr(self, "protocolclass", 0) and \
-####                getattr(self.protocolclass, "BREW_FILE_SYSTEM", 0) == 2:
-####            print '_set_new_brew', self.protocolclass
-####            self._set_new_brew(self.__class__)
-##
-##    def __call__(self, *args, **kwargs):
-##        return self._klass.__call__(self, *args, **kwargs)
-##    def _printclass(self, klass):
-##        if klass.__bases__:
-##            for _c in klass.__bases__:
-##                self._printclass(_c)
-##        else:
-##            print klass
-##            
-##    def _update_base_class(self, klass):
-##        # update the RealBrewProtocol class to DebugBrewProtocol one.
-##        _bases=[]
-##        found=False
-##        for e in klass.__bases__:
-##            if e==RealBrewProtocol or e==RealBrewProtocol2:
-##                _bases.append(DebugBrewProtocol)
-##                found=True
-##            else:
-##                _bases.append(e)
-##        if found:
-##            klass.__bases__=tuple(_bases)
-##        else:
-##            for e in _bases:
-##                self._update_base_class(e)
-##
-##    def _set_new_brew(self, klass):
-##        # update the class hierachy to include RealBrewProtocol2 so that 
-##        # it's functions override RealBrewProtocol's ones.
-##        _bases=[]
-##        found=False
-##        for e in klass.__bases__:
-##            if e==RealBrewProtocol:
-##                _bases.append(RealBrewProtocol2)
-##                found=True
-##            _bases.append(e)
-##        if found:
-##            klass.__bases__=tuple(_bases)
-##        else:
-##            for e in _bases:
-##                self._set_new_brew(e)
-
 def formatpacketerrorlog(str, origdata, data, klass):
     # copied from guiwidgets.LogWindow.logdata
     hd=""
@@ -1278,6 +1268,73 @@ class SPURIOUSZERO(prototypes.BaseProtogenClass):
              
          self._bufferendoffset=buf.getcurrentoffset()
 
+class EXTRAZERO(prototypes.BaseProtogenClass):
+    """This is a special class used to consume the spurious zero in some p_brew.listfileresponse or p_brew.listdirectoryresponse
+
+    The two bytes are formatted as follows:
+
+       - An optional 'null' byte (this class)
+       - A byte specifying the length of the whole name
+       - The bytes of the filename (which includes the full directory name)
+
+    Allow for zero length filenames.
+    
+    """
+    def __init__(self, *args, **kwargs):
+        super(EXTRAZERO,self).__init__(*args, **kwargs)
+        
+        self._value=None
+        if self._ismostderived(EXTRAZERO):
+            self._update(args, kwargs)
+
+    def _update(self, args, kwargs):
+        super(EXTRAZERO, self)._update(args, kwargs)
+        
+        self._complainaboutunusedargs(EXTRAZERO, kwargs)
+
+        if len(args):
+            raise TypeError("Unexpected arguments "+`args`)
+
+    def readfrombuffer(self, buf):
+         self._bufferstartoffset=buf.getcurrentoffset()
+
+         # there are several cases this code has to deal with
+         #
+         # The data is ordered like this:
+         #
+         # optional spurious zero (sz)
+         # fulllen
+         # name
+         #
+         # These are the various possibilities.  The first two
+         # are a file in the root directory (dirlen=0), with the other
+         # two being a file in a subdirectory  (dirlen>0). fulllen
+         # is always >0
+         #
+         # A:    fulllen=0
+         # B: ez fulllen=0
+         # C:    fulllen>0 name
+         # D: ez fulllen>0 name
+
+         while True:  # this is just used so we can break easily
+
+             # CASE C
+             if buf.peeknextbyte()!=0:
+                 self._value=-1
+                 break
+             
+             # CASE A
+             if buf.howmuchmore()==1:
+                 self._value=-1
+                 break # Really a zero length file
+
+             # CASE B or D
+             self._value=buf.getnextbyte() # consume sz
+
+             break
+             
+         self._bufferendoffset=buf.getcurrentoffset()
+
     def writetobuffer(self, buf):
         raise NotImplementedError()
 
