@@ -6,10 +6,11 @@
 ### This program is free software; you can redistribute it and/or modify
 ### it under the terms of the BitPim license as detailed in the LICENSE file.
 ###
-### $Id: com_lg.py 4456 2007-11-24 05:14:27Z djpham $
+### $Id: com_lg.py 4693 2008-08-15 22:30:51Z djpham $
 
 """Phonebook conversations with LG phones"""
 
+import functools
 import threading
 
 import com_brew
@@ -17,6 +18,7 @@ import com_phone
 import p_lg
 import prototypes
 import common
+import struct
 
 class LGPhonebook:
 
@@ -1323,7 +1325,8 @@ class LGUncountedIndexedMedia:
         dircache.rmfile(test_name)
         return True
 
-    def savemedia(self, mediakey, mediaindexkey, maps, results, merge, reindexfunction):
+    def savemedia(self, mediakey, mediaindexkey, maps, results, merge,
+                  reindexfunction, update_index_file=True):
         """Actually saves out the media
 
         @param mediakey: key of the media (eg 'wallpapers' or 'ringtones')
@@ -1459,25 +1462,26 @@ class LGUncountedIndexedMedia:
                     else:
                         dircache.writefile(filename, data)
             # write out index
-            ifile=self.protocolclass.indexfile()
-            idxlist=init[type].keys()
-            idxlist.sort()
-            idxlist.reverse()
-            for idx in idxlist:
-                ie=self.protocolclass.indexentry()
-                ie.type=typemajor
-                ie.filename=init[type][idx]['filename']
-                fstat=dircache.stat(init[type][idx]['filename'])
-                if fstat is not None:
-                    ie.size=fstat['size']
-                else:
-                    ie.size=0
-                # ie.date left as zero
-                ifile.items.append(ie)
-            buf=prototypes.buffer()
-            ifile.writetobuffer(buf, logtitle="Index file "+indexfile)
-            self.log("Writing index file "+indexfile+" for type "+type+" with "+`len(idxlist)`+" entries.")
-            dircache.writefile(indexfile, buf.getvalue()) # doesn't really need to go via dircache
+            if update_index_file:
+                ifile=self.protocolclass.indexfile()
+                idxlist=init[type].keys()
+                idxlist.sort()
+                idxlist.reverse()
+                for idx in idxlist:
+                    ie=self.protocolclass.indexentry()
+                    ie.type=typemajor
+                    ie.filename=init[type][idx]['filename']
+                    fstat=dircache.stat(init[type][idx]['filename'])
+                    if fstat is not None:
+                        ie.size=fstat['size']
+                    else:
+                        ie.size=0
+                    # ie.date left as zero
+                    ifile.items.append(ie)
+                buf=prototypes.buffer()
+                ifile.writetobuffer(buf, logtitle="Index file "+indexfile)
+                self.log("Writing index file "+indexfile+" for type "+type+" with "+`len(idxlist)`+" entries.")
+                dircache.writefile(indexfile, buf.getvalue()) # doesn't really need to go via dircache
         return reindexfunction(results)
                             
     def getwallpaperindices(self, results):
@@ -1603,16 +1607,79 @@ class LGDMPhone:
         if _resp.unlock_ok!=1:
             raise EnterDMError('Bad response - unlock_ok: %d'%_resp.unlock_ok)
 
+    def _DMv6_get_esn(self):
+        _req=self.protocolclass.NVReq(field=0x0000)
+        _res=self.sendbrewcommand(_req, self.protocolclass.NVRes)
+
+        return struct.unpack_from ('<L', _res.data, 0)[0]
+
+    def _DMv6_get_extra_data(self,data_field):
+        _req=self.protocolclass.NVReq(field=data_field)
+        _res=self.sendbrewcommand(_req, self.protocolclass.NVRes)
+
+        return struct.unpack_from ('<L', _res.data, 0)[0]
+
+    def _DMv6_get_compile_time(self):
+        _req=self.protocolclass.FWInfoReq()
+        _res=self.sendbrewcommand(_req, self.protocolclass.FWInfoRes)
+
+        return _res.get_compile_time()
+
+    def _enter_DMv6(self):
+        # this loop is a hack -- different LG phones use different commands to get _extra. once
+        # the command is figured out we won't have to try every possible shift.
+        if self._shift is None:
+            _shifts=range(4)
+        else:
+            _shifts=[self._shift]
+        for _shift in _shifts:
+            # similar but slightly different from v5, the enV2 started this!
+            # request the seed
+            _req=self.protocolclass.DMKeyReq()
+            _resp=self.sendbrewcommand(_req, self.protocolclass.DMKeyResp)
+            _key=self.get_challenge_response(_resp.unlock_key)
+            if _key is None:
+                self.log('Failed to get the key.')
+                raise EnterDMError('Failed to get the key')
+
+            #_esn=self._DMv6_get_esn()
+            #_extra=self._DMv6_get_extra_data(0x13d7) # VX-9100 uses data field 0x13d7
+            #_ctime=self._DMv6_get_compile_time()
+
+            # determine how many bytes the key needs be be shifted
+            #_shift_bytes = (_esn + ((_extra >> 16) & 0xf) + ((_extra >> 11) & 0x1f) + _ctime) % 16
+            #_shift = _shift_bytes / 4
+
+            _req=self.protocolclass.DMEnterReq(unlock_key=_key)
+            if _resp.unlock_code==0:
+                # this response usually occurs in error. rebooting the phone seems to clear it
+                _req.unlock_code=1
+            elif _resp.unlock_code==2:
+                _req.unlock_code=3
+                _req.convert_to_key2(_shift)
+            else:
+                raise EnterDMError('Unknown unlock_code: %d'%_resp.unlock_code)
+            _resp=self.sendbrewcommand(_req, self.protocolclass.DMEnterResp)
+            if _resp.result == 1:
+                if self._shift is None:
+                    self._shift=_shift
+                    if __debug__:
+                        self.log('Shift key: %d'%self._shift)
+                return
+        raise EnterDMError('Failed to guess the shift/key')
+
     def enter_DM(self, e=None):
-        # do nothing if (1) the phone is already in DM, or (2) the phone failed
-        # to previously enter DM
-        if self._in_DM is not None:
+        # do nothing if the phone failed to previously enter DM
+        if self._in_DM is False:
             return
         # check for DMv5 applicability
         if self._DMv5 is None:
             self.setDMversion()
         try:
-            if self._DMv5:
+            if self._DMv6:
+                # new DM scheme
+                self._enter_DMv6()
+            elif self._DMv5:
                 # enter DMv5
                 self._enter_DMv5()
             else:
@@ -1622,21 +1689,7 @@ class LGDMPhone:
         except:
             self.log('Failed to transition to DM')
             self._in_DM=False
-            if __debug__:
-                raise
             return
-        
-        self.log('Successfully transitioned to DM')
-        # DM entered successfully, kick off the timer if specified
-        if self._timeout:
-            # clear existing timer
-            if self._timer:
-                self._timer.cancel()
-                del self._timer
-            # and set a new one
-            self.log('Starting DM timeout for %s seconds'%self._timeout)
-            self._timer=threading.Timer(self._timeout, self._OnTimer)
-            self._timer.start()
 
     def _OnTimer(self):
         if self._in_DM:
@@ -1645,35 +1698,44 @@ class LGDMPhone:
         del self._timer
         self._timer=None
 
-    def _getfilecontents(self, name, use_cache=False):
+    def _filefunc(self, func, *args, **kwargs):
         self.enter_DM()
-        return self._real_getfilecontents(name, use_cache)
+        return func(*args, **kwargs)
 
-    def _writefile(self, name, contents):
+    def _sendbrewcommand(self, func, *args, **kwargs):
+        # A wrapper function to address the issue of DM timing out in a
+        # middle of a file read or write (usually of a large file).
+        # Not quite happy with this hack, but couldn't figure out a better way!
+        # If you have a better solution, feel free to share.
+        try:
+            return func(*args, **kwargs)
+        except com_brew.BrewAccessDeniedException:
+            # DM may have timed out, retry.
+            pass
         self.enter_DM()
-        return self._real_writefile(name, contents)
-                            
-    def _statfile(self, name):
-        self.enter_DM()
-        return self._real_statfile(name)
+        return func(*args, **kwargs)
 
     def setDMversion(self):
         """Define the DM version required for this phone, default to DMv5"""
         self._DMv5=True
+        self._DMv6=False
 
     def __init__(self):
         self._in_DM=None
         self._timer=None
         self._DMv5=None
+        self._DMv6=None
         self._timeout=None
-        if not hasattr(self, '_real_getfilecontents'):
-            (self._real_getfilecontents,
-             self.getfilecontents)=(self.getfilecontents,
-                                   self._getfilecontents)
-            (self._real_writefile,
-             self.writefile)=(self.writefile, self._writefile)
-            (self._real_statfile,
-             self.statfile)=(self.statfile, self._statfile)
+        self._shift=None
+        # wrap our functions
+        self.getfilecontents=functools.partial(self._filefunc,
+                                               self.getfilecontents)
+        self.writefile=functools.partial(self._filefunc,
+                                         self.writefile)
+        self.statfile=functools.partial(self._filefunc,
+                                        self.statfile)
+        self.sendbrewcommand=functools.partial(self._sendbrewcommand,
+                                               self.sendbrewcommand)
 
     def __del__(self):
         if self._timer:
