@@ -5,7 +5,7 @@
 ### This program is free software; you can redistribute it and/or modify
 ### it under the terms of the BitPim license as detailed in the LICENSE file.
 ###
-### $Id: com_phone.py 4543 2008-01-04 23:44:24Z djpham $
+### $Id: com_phone.py 4636 2008-07-21 03:25:38Z djpham $
 
 
 """Generic phone stuff that all models inherit from"""
@@ -18,6 +18,7 @@ import field_color
 import re
 import sys
 import time
+import prototypes
 
 
 # when trying to setmode, we ignore various exception types
@@ -138,7 +139,29 @@ class Phone(object):
                 return True
             except modeignoreerrortypes:
                 pass
-        return False       
+        return False
+
+    def readobject(self, filename, object_class, logtitle=None,
+                   uselocalfs=False):
+        """Read the specified filename and bind it to the object class"""
+        if uselocalfs:
+            self.log('Reading local file: %s'%filename)
+            _buf=prototypes.buffer(file(filename, 'rb').read())
+        else:
+            _buf=prototypes.buffer(self.getfilecontents(filename))
+        _obj=object_class()
+        _obj.readfrombuffer(_buf, logtitle=logtitle)
+        return _obj
+
+    def writeobject(self, filename, obj, logtitle=None,
+                    uselocalfs=False):
+        """Writhe the object into the file"""
+        _buf=prototypes.buffer()
+        obj.writetobuffer(_buf, logtitle=logtitle)
+        if uselocalfs:
+            file(filename, 'wb').write(_buf.getvalue())
+        else:
+            self.writefile(filename, _buf.getvalue())
 
     getmemo=NotImplemented
     gettodo=NotImplemented
