@@ -6,7 +6,7 @@
 ### This program is free software; you can redistribute it and/or modify
 ### it under the terms of the BitPim license as detailed in the LICENSE file.
 ###
-### $Id: com_samsung_packet.py 3634 2006-10-29 22:22:57Z sawecw $
+### $Id: com_samsung_packet.py 4470 2007-11-28 04:27:52Z djpham $
 
 """Communicate with a Samsung SCH-Axx phone using AT commands"""
 
@@ -181,12 +181,31 @@ class Phone(com_phone.Phone, com_brew.BrewProtocol):
         req=self.protocolclass.esnrequest()
         res=self.sendpbcommand(req, self.protocolclass.esnresponse)
         try:
-            print res[0].esn
             return res[0].esn
         except:
             pass
         return ''
-
+    def get_model(self):
+        req=self.protocolclass.modelreq()
+        res=self.sendpbcommand(req, self.protocolclass.modelresp)
+        try:
+            return res[0].model
+        except:
+            return ''
+    def get_manufacturer(self):
+        req=self.protocolclass.manufacturerreq()
+        res=self.sendpbcommand(req, self.protocolclass.manufacturerresp)
+        try:
+            return res[0].manufacturer
+        except:
+            return ''
+    def get_battery_level(self):
+        req=self.protocolclass.batterylevelreq()
+        res=self.sendpbcommand(req, self.protocolclass.batterylevelresp)
+        try:
+            return res[0].levelstr
+        except:
+            return ''
     def read_groups(self):
 
         g={}
@@ -250,7 +269,11 @@ class Phone(com_phone.Phone, com_brew.BrewProtocol):
                 entry=self.extractphonebookentry(res[0].entry, result)
                 pbook[count]=entry
                 count+=1
-            self.progress(slot, self.protocolclass.NUMPHONEBOOKENTRIES, lastname)
+            else:
+                lastname=""
+            self.progress(slot, self.protocolclass.NUMPHONEBOOKENTRIES,
+                          'Reading entry %(slot)d: %(name)s'%{ 'slot': slot,
+                                                               'name': lastname })
         
         result['phonebook']=pbook
         cats=[]
@@ -261,26 +284,9 @@ class Phone(com_phone.Phone, com_brew.BrewProtocol):
         print "returning keys",result.keys()
         
         return pbook
-        
-    def extractphonebookentry(self, entry, fundamentals):
-        res={}
-
-        res['serials']=[ {'sourcetype': self.serialsname,
-                          'slot': entry.slot,
-                          'sourceuniqueid': fundamentals['uniqueserial']} ]
-        # only one name
-        res['names']=[ {'full': entry.name} ]
-        # only one category
-        cat=fundamentals['groups'].get(entry.group, {'name': "Unassigned"})['name']
-        if cat!="Unassigned":
-            res['categories']=[ {'category': cat} ]
-        # only one email
-        if len(entry.email):
-            res['emails']=[ {'email': entry.email} ]
-        # only one url
-        if len(entry.url):
-            res['urls']=[ {'url': entry.url} ]
 
+    def _extractphonebook_numbers(self, entry, fundamentals, res):
+        """Extract and build phone numbers"""
         res['numbers']=[]
         secret=0
 
@@ -300,19 +306,44 @@ class Phone(com_phone.Phone, com_brew.BrewProtocol):
         # Field after each number is secret flag.  Setting secret on
         # phone sets secret flag for every defined phone number
         res['flags']=[ {'secret': secret} ]
-
+    def _extractphonebook_ringtone(self, entry, fundamentals, res):
+        """Extract ringtone info"""
         if entry.ringtone != self.protocolclass.DEFAULT_RINGTONE:
             tone=self.serialsname+"Index_"+`entry.ringtone`
             res['ringtones']=[{'ringtone': tone, 'use': 'call'}]
-            
+    def _extractphonebook_wallpaper(self, entry, fundamentals, res):
+        """Extract wallpaper info"""
         try:
             if entry.wallpaper != self.protocolclass.DEFAULT_WALLPAPER:
                 tone=self.serialsname+"Index_"+`entry.wallpaper`
                 res['wallpapers']=[{'wallpaper': tone, 'use': 'call'}]
-
         except:
             pass
 
+    def extractphonebookentry(self, entry, fundamentals):
+        res={}
+
+        res['serials']=[ {'sourcetype': self.serialsname,
+                          'slot': entry.slot,
+                          'sourceuniqueid': fundamentals['uniqueserial']} ]
+        # only one name
+        res['names']=[ {'full': entry.name} ]
+        # only one category
+        cat=fundamentals['groups'].get(entry.group, {'name': "Unassigned"})['name']
+        if cat!="Unassigned":
+            res['categories']=[ {'category': cat} ]
+        # only one email
+        if len(entry.email):
+            res['emails']=[ {'email': entry.email} ]
+        # only one url
+        if len(entry.url):
+            res['urls']=[ {'url': entry.url} ]
+        # separate the following processing into methods so subclass can
+        # customize them
+        self._extractphonebook_numbers(entry, fundamentals, res)
+        self._extractphonebook_ringtone(entry, fundamentals, res)
+        self._extractphonebook_wallpaper(entry, fundamentals, res)
+
         # We don't have a place to put these
         # print entry.name, entry.birthday
         # print entry.name, entry.timestamp
@@ -370,8 +401,6 @@ class Phone(com_phone.Phone, com_brew.BrewProtocol):
             slot=keys[i]
             req=self.protocolclass.phonebookslotupdaterequest()
             req.entry=self.makeentry(pb[slot],data)
-            req.entry.ringtone=self.protocolclass.DEFAULT_RINGTONE
-            req.entry.wallpaper=self.protocolclass.DEFAULT_WALLPAPER
             if names[slot]==req.entry.name:
                 req.entry.birthday=birthdays[slot]
             self.log('Writing entry '+`slot`+" - "+req.entry.name)
@@ -406,6 +435,8 @@ class Phone(com_phone.Phone, com_brew.BrewProtocol):
                 continue
             # everything else we just set
             setattr(e, k, entry[k])
+        e.ringtone=self.protocolclass.DEFAULT_RINGTONE
+        e.wallpaper=self.protocolclass.DEFAULT_WALLPAPER
         return e
 
     def getcalendar(self, result):
@@ -628,12 +659,12 @@ class Phone(com_phone.Phone, com_brew.BrewProtocol):
             
             req.slot=todo_cnt
             if todo.priority is not None and todo.priority<5:
-                req.priority=0
-            else:
                 req.priority=1
+            else:
+                req.priority=0
                 
             dd=todo.due_date
-            req.duedate=(int(dd[0:3]),int(dd[4:5]),int(dd[6:7]),0,0,0)
+            req.duedate=(int(dd[:4]),int(dd[4:6]),int(dd[6:10]),0,0,0)
             req.timestamp=list(time.localtime(time.time())[0:6])
             req.subject=todo.summary
             self.sendpbcommand(req,self.protocolclass.todoupdateresponse)
@@ -680,6 +711,7 @@ class Phone(com_phone.Phone, com_brew.BrewProtocol):
             dd=memo.set_date_isostr
             req.timestamp=list(time.localtime(time.time())[0:6])
             req.text=memo.text
+            req.slot=memo_cnt
             self.sendpbcommand(req,self.protocolclass.memoupdateresponse)
             memo_cnt += 1
 
@@ -688,7 +720,15 @@ class Phone(com_phone.Phone, com_brew.BrewProtocol):
             req.slot=slot
             self.sendpbcommand(req,self.protocolclass.memoupdateresponse)
 
-    
+    # return some basic info about this phone
+    def getbasicinfo(self, phoneinfo):
+        self.log('Getting Basic Phone Info')
+        for _key,_ in phoneinfo.standard_keys:
+            _val=getattr(self, 'get_'+_key,
+                         lambda *_: '')()
+            if _val:
+                setattr(phoneinfo, _key, _val)
+    getphoneinfo=getbasicinfo        
     getcallhistory=None
         
 class Profile(com_phone.Profile):
