@@ -5,11 +5,13 @@
 ### This program is free software; you can redistribute it and/or modify
 ### it under the terms of the BitPim license as detailed in the LICENSE file.
 ###
-### $Id: vcal_calendar.py 4179 2007-04-03 01:07:57Z djpham $
+### $Id: vcal_calendar.py 4396 2007-09-12 21:44:00Z djpham $
 
 "Deals with vcard calendar import stuff"
 
 # system modules
+from __future__ import with_statement
+import contextlib
 import copy
 import datetime
 
@@ -20,6 +22,7 @@ import wx
 import bpcalendar
 import bptime
 import common_calendar
+import guihelper
 import helpids
 import vcard
 
@@ -79,19 +82,18 @@ class vCalendarFile(object):
             # no file name specified
             return
         try:
-            f=self._open(self._file_name)
-            vfile=vcard.VFile(f)
-            has_data=False
-            for n,l in vfile:
-                if n[0]=='BEGIN' and l=='VEVENT':
-                    has_data=True
-                    _vdata=[]
-                elif n[0]=='END' and l=='VEVENT':
-                    has_data=False
-                    self._data.append(self._read_block(_vdata))
-                elif has_data:
-                    _vdata.append((n, l))
-            f.close()
+            with contextlib.closing(self._open(self._file_name)) as f:
+                vfile=vcard.VFile(f)
+                has_data=False
+                for n,l in vfile:
+                    if n[0]=='BEGIN' and l=='VEVENT':
+                        has_data=True
+                        _vdata=[]
+                    elif n[0]=='END' and l=='VEVENT':
+                        has_data=False
+                        self._data.append(self._read_block(_vdata))
+                    elif has_data:
+                        _vdata.append((n, l))
         except:
             if __debug__:
                 raise
@@ -190,6 +192,7 @@ class VCalendarImportData(object):
         else:
             # not yet supported
             return
+        rp.weekstart=e.get('repeat_wkst', 'MO')
         # setting the repeat duration/end-date of this event
         if rp_end is not None:
             # end date specified
@@ -557,39 +560,36 @@ class VcalImportCalDialog(common_calendar.PreviewDialog):
         wx.EVT_BUTTON(self, self.ID_MERGE, self.OnEndModal)
         wx.EVT_BUTTON(self, wx.ID_HELP, lambda *_: wx.GetApp().displayhelpid(helpids.ID_DLG_CALENDAR_IMPORT))
 
+    @guihelper.BusyWrapper
     def OnImport(self, evt):
-        wx.BeginBusyCursor()
-        dlg=wx.ProgressDialog('%s Import'%self._data_type,
-                              'Importing %s Data, please wait ...'%self._data_type,
-                              parent=self)
-        try:
-            self._oc.read(self.folderctrl.GetValue())
-            self.populate(self._oc.get_display_data())
-        except (ValueError, IOError):
-            _err_dlg=wx.MessageDialog(self, 'Failed to get import data',
-                                      'Import Error',
-                                      style=wx.OK|wx.ICON_ERROR)
-            _err_dlg.ShowModal()
-            _err_dlg.Destroy()
-        except:
-            if __debug__:
-                raise
-        dlg.Destroy()
-        wx.EndBusyCursor()
+        with guihelper.WXDialogWrapper(wx.ProgressDialog('%s Import'%self._data_type,
+                                                         'Importing %s Data, please wait ...'%self._data_type,
+                                                         parent=self)) as dlg:
+            try:
+                self._oc.read(self.folderctrl.GetValue())
+                self.populate(self._oc.get_display_data())
+            except (ValueError, IOError):
+                guihelper.MessageDialog(self, 'Failed to get import data',
+                                        'Import Error',
+                                        style=wx.OK|wx.ICON_ERROR)
+            except:
+                if __debug__:
+                    raise
 
     def OnBrowseFolder(self, evt):
-        dlg=wx.FileDialog(self, "Pick a %s File"%self._data_type,
-                          wildcard='*.vcs;*.ics')
-        if dlg.ShowModal()==wx.ID_OK:
-            self.folderctrl.SetValue(dlg.GetPath())
-        dlg.Destroy()
+        with guihelper.WXDialogWrapper(wx.FileDialog(self, "Pick a %s File"%self._data_type,
+                                                     wildcard='*.vcs;*.ics'),
+                                       True) as (dlg, retcode):
+            if retcode==wx.ID_OK:
+                self.folderctrl.SetValue(dlg.GetPath())
 
     def OnFilter(self, evt):
         cat_list=self._oc.get_category_list()
-        dlg=common_calendar.FilterDialog(self, -1, 'Filtering Parameters', cat_list)
-        if dlg.ShowModal()==wx.ID_OK:
-            self._oc.set_filter(dlg.get())
-            self.populate(self._oc.get_display_data())
+        with guihelper.WXDialogWrapper(common_calendar.FilterDialog(self, -1, 'Filtering Parameters', cat_list),
+                                       True) as (dlg, retcode):
+            if retcode==wx.ID_OK:
+                self._oc.set_filter(dlg.get())
+                self.populate(self._oc.get_display_data())
 
     def OnEndModal(self, evt):
         self.EndModal(evt.GetId())
@@ -645,14 +645,11 @@ class VCalAutoConfCalDialog(wx.Dialog):
         main_bs.Fit(self)
 
     def OnBrowseFolder(self, evt):
-        dlg=wx.FileDialog(self, "Pick a VCalendar File", wildcard='*.vcs')
-        id=dlg.ShowModal()
-        if id==wx.ID_CANCEL:
-            dlg.Destroy()
-            return
-        self.folderctrl.SetValue(dlg.GetPath())
-        self._read=False
-        dlg.Destroy()
+        with guihelper.WXDialogWrapper(wx.FileDialog(self, "Pick a VCalendar File", wildcard='*.vcs'),
+                                       True) as (dlg, retcode):
+            if retcode==wx.ID_OK:
+                self.folderctrl.SetValue(dlg.GetPath())
+                self._read=False
 
     def OnFilter(self, evt):
         # read the calender to get the category list
@@ -660,10 +657,11 @@ class VCalAutoConfCalDialog(wx.Dialog):
             self._oc.read(self.folderctrl.GetValue())
             self._read=True
         cat_list=self._oc.get_category_list()
-        dlg=common_calendar.AutoSyncFilterDialog(self, -1, 'Filtering Parameters', cat_list)
-        dlg.set(self._oc.get_filter())
-        if dlg.ShowModal()==wx.ID_OK:
-            self._oc.set_filter(dlg.get())
+        with guihelper.WXDialogWrapper(common_calendar.AutoSyncFilterDialog(self, -1, 'Filtering Parameters', cat_list)) \
+             as dlg:
+            dlg.set(self._oc.get_filter())
+            if dlg.ShowModal()==wx.ID_OK:
+                self._oc.set_filter(dlg.get())
 
     def GetFolder(self):
         return self.folderctrl.GetValue()
